package algorithmes;

import graphe.*;
import java.util.Stack;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.Comparator;


public abstract class AlgoDeGraphes
{
	/* Dijsktra */
	public static void dijsktra(UnGraphe telG, int telD, int telA)
	{
		int[] lePere=new int[telG.donneNbSommets()];
		Stack laPilePCC=new Stack();
		int s;
		UnVoisin leCourant;
		
		initialisation_pere(lePere, telD);
		
		
		while(telG.resteSommetNonMarque())
		{
			s=donneSommetNonMarqueMin(telG, lePere);
			telG.donneSommet(s).marqueSommet();
			leCourant=telG.donneSommet(s).donneSaListeVoisins();
			
			
			while(leCourant!=null)
			{
				if(lePere[leCourant.donneSonIndice()]>
											leCourant.donneSaValeur()+lePere[s])
				{
					lePere[leCourant.donneSonIndice()]=
											leCourant.donneSaValeur()+lePere[s];
				}
				
				leCourant=leCourant.donneSuivant();
			}
		}
		
		
		/* lePere est rempli pour chaque sommet, du poids total du plus court 
		 * chemin.
		 */
		laPilePCC.push(new Integer(telA));
		s=telA;
		while(s!=telD)
		{
			leCourant=telG.donneSommet(s).donneSaListeVoisins();
			while(leCourant!=null)
			{
				if(lePere[s]-leCourant.donneSaValeur()==
										lePere[leCourant.donneSonIndice()])
				{
					laPilePCC.push(new Integer(leCourant.donneSonIndice()));
					s=leCourant.donneSonIndice();
					leCourant=null;
				}
				else
				{
					leCourant=leCourant.donneSuivant();
				}
			}
		}
	 
		//Affichage du resultat
		while(!laPilePCC.empty())
		{
			System.out.print(
			telG.donneNomSommet(((Integer)laPilePCC.pop()).intValue())+
			" - ");
		}
		
		System.out.println ();
	}
	
	private static void initialisation_pere(int[] telPere, int telD)
	{
		for(int i=0; i<telPere.length; i++)
		{
			if(i==telD)
			{
				telPere[i]=0;
			}
			else
			{
				telPere[i]=Constants.INFINI;
			}
		}
	}
	
	
	private static int donneSommetNonMarqueMin(UnGraphe telG, int[] telPere)
	{
		int leMin=0;
		int laVal=Integer.MAX_VALUE;
		
		for(int i=0; i<telPere.length; i++)
		{
			if(telPere[i]!=Constants.INFINI && 
				!telG.donneSommet(i).estMarque() &&
				telPere[i]<laVal)
			{
				leMin=i;
				laVal=telPere[i];
			}
		}
		
		return leMin;
	}
	
	/* ------------------------ */
	
	/* Kruskall */
	public static void kruskall(UnGraphe telG)
	{
		int[] leT=new int[telG.donneNbSommets()];
		List laListe=donneAretesTrie(telG);
		int i;
		UnGraphe leGPrime=new UnGraphe(telG.donneNbSommets());
		leGPrime.sesNomsSommets=telG.sesNomsSommets;
		
		for(i=0; i<leT.length; i++)
		{
			leT[i]=i;
		}
		
		i=0;
		while(i<laListe.size())
		{
			if( leT[((UneArete)laListe.get(i)).donneDepart()]!=
				leT[((UneArete)laListe.get(i)).donneArrivee()])
			{
				mettreAJourT(leT, ((UneArete)laListe.get(i)).donneDepart(),
								  ((UneArete)laListe.get(i)).donneArrivee());
				leGPrime.ajouteVoisin(
									((UneArete)laListe.get(i)).donneDepart(),
									((UneArete)laListe.get(i)).donneArrivee(),
									((UneArete)laListe.get(i)).donnePoids());
				leGPrime.ajouteVoisin(
									((UneArete)laListe.get(i)).donneArrivee(),
									((UneArete)laListe.get(i)).donneDepart(),
									((UneArete)laListe.get(i)).donnePoids());
			}
			
			i++;
		}
		
		leGPrime.afficheToi();
	}
	
	private static List donneAretesTrie(UnGraphe telG)
	{
		List laListe=new Vector();
		UnVoisin leCourant=null;
		
		//Creation de la liste
		for(int i=0; i<telG.donneNbSommets(); i++)
		{
			leCourant=telG.donneSommet(i).donneSaListeVoisins();
			
			while(leCourant!=null)
			{
				if(leCourant.donneSonIndice()>i)
				{
					laListe.add(new UneArete(i, leCourant.donneSonIndice(), 
											leCourant.donneSaValeur()));
				}
				
				leCourant=leCourant.donneSuivant();
			}
		}
		
		Collections.sort(laListe);
		
		return laListe;
	}
	
	private static void mettreAJourT(int[] telT, int telD, int telA)
	{
		int laVal, j;
		
		j=0;
		laVal=telT[telA];
		
		while(j<telT.length)
		{
			if(telT[j]==laVal)
			{
				telT[j]=telT[telD];
			}
			
			j++;
		}
	}
	
	/* ------------------------ */
}